<?php

App::uses('GatewayBaseComponent', 'EvTransactions.Controller/Component');
App::uses('GatewayInterface', 'EvTransactions.Lib');
App::uses('CakeText', 'Utility');

/**
 * flow and processes used here were taken from
 * @link http://paypal.github.io/PayPal-PHP-SDK/sample/doc/payments/CreatePaymentUsingPayPal.html
 * @link http://paypal.github.io/PayPal-PHP-SDK/sample/doc/payments/ExecutePayment.html
 */
class PaypalComponent extends GatewayBaseComponent implements GatewayInterface {

	/**
	 * loaded API context
	 */
	protected $_apiContext = null;

	/**
	 * object storage
	 */
	protected $_objects = array();

	/**
	 * apiContext - setup oauth connection when requested
	 *
	 * @return 	PayPal\Rest\ApiContext
	 */
	protected function _apiContext() {
		if ($this->_apiContext !== null) {
			return $this->_apiContext;
		}

		$env = Configure::read('app.environment');

		if ($env == 'PRODUCTION') {
			$this->_config = Configure::read('EvPaypal.live');
		} else {
			$this->_config = Configure::read('EvPaypal.dev');
		}

		$this->_apiContext = new ApiContext(
			new OAuthTokenCredential(
				$this->_config['clientId'],
				$this->_config['clientSecret']
			)
		);

		// @link https://developer.paypal.com/webapps/developer/docs/api/#inputfields-object
		$this->_apiContext->setConfig(
			array(
				'no_shipping' => $this->_config['noShipping']
			)
		);

		if ($this->_config['testMode']) {
			$this->_apiContext->setConfig(
				array(
					'mode' => 'sandbox'
				)
			);
		}

		// @todo setup web profile for logo / brand name

		return $this->_apiContext;
	}

	/**
	 * init
	 *
	 * Use the function setup and connection / settings that need setting up in the gateway
	 */
	public function setup() {
		$this->_objects['payer'] = new \PayPal\Api\Payer;
		$this->_objects['payer']->setPaymentMethod('paypal');

		$this->_objects['payment'] = new \PayPal\Api\Payment;
		$this->_objects['payment']->setIntent('sale')
			->setPayer($this->_objects['payer']);
	}

	/**
	 * setupPayment
	 * @todo ADDRESSES
	 *
	 * Use this function setup the actual payment, i.e. setup the basket, the amount to take etc...
	 *
	 * @param int - transaction id we have created
	 * @param array $return - array with 2 keys of 'return', 'cancel'. Containing either a link or router array for redirecting user
	 * @param array $model - array with two keys of 'model' and 'model_id', used to link transactions polymorphically to other model items
	 * @param float|array $amount - amount of monies to take (I GOT YOUR MONIESSSSSS), or array of 'amount' and 'currency' to change currencies (will take default from config)
	 * @param array $items - multidimenisional array break down of the transaction items, required elements are 'description' and 'amount'
	 * @param mixed $extra - variable allowing you to pass ay data needed to the gateway, could be things like addresses that are not tracked by the transactions model
	 */
	public function setupPayment($transactionId, $return, $model, $amount, $items, $extra = array()) {
		// process the maount var to get array of amount / currency

		$amount = $this->processAmount($amount);
		$PaypalAmount = new \PayPal\Api\Amount;
		$PaypalAmount->setCurrency($amount['currency'])
			->setTotal($amount['amount']);

		// check for extra totals
		if (! empty($extra['totals'])) {
			$PaypalDetails = new \PayPal\Api\Details;

			if (! empty($extra['totals']['shipping'])) {
				$PaypalDetails->setShipping($extra['totals']['shipping']);
			}

			if (! empty($extra['totals']['tax'])) {
				$PaypalDetails->setTax($extra['totals']['tax']);
			}

			if (! empty($extra['totals']['subtotal'])) {
				$PaypalDetails->setSubtotal($extra['totals']['subtotal']);
			}

			$PaypalAmount->setDetails($PaypalDetails);
		}

		// setup description
		$description = 'Payment to ' . $this->_config['brandname'];
		// Override generic description with custom one if exists
		if (isset($extra['description'])) {
			$description = $extra['description'];
		}

		// loop transaction items
		$PaypalItemList = new \PayPal\Api\ItemList;

		if (! empty($items)) {
			foreach ($items as $item) {
				// check it has all the required fields
				if (isset($item['quantity']) && isset($item['unitPrice'])) {
					$PaypalItem = new \PayPal\Api\Item;
					$PaypalItem->setName($item['description'])
						->setQuantity($item['quantity'])
						->setPrice($item['unitPrice'])
						->setCurrency($amount['currency']);

					if (! empty($item['taxAmount'])) {
						$PaypalItem->setTax($item['taxAmount']);
					}

					$PaypalItemList->addItem($PaypalItem);
				}
			}
		}

		// Setup the transaction
		$PaypalTransaction = new \PayPal\Api\Transaction;
		$PaypalTransaction->setAmount($PaypalAmount)
			->setDescription($description)
			->setInvoiceNumber(CakeText::uuid())
			->setItemList($PaypalItemList);

		// setup redirect urls
		$PaypalUrls = new \PayPal\Api\RedirectUrls;

		if (! empty($return['return'])) {
			$PaypalUrls->setReturnUrl($return['return']);
		}

		if (! empty($return['cancel'])) {
			$PaypalUrls->setCancelUrl($return['cancel']);
		}

		// set the transaction and redirect urls to the payment object so it's ready to create
		$this->_objects['payment']->setRedirectUrls($PaypalUrls)
			->setTransactions(array($PaypalTransaction));
	}

	/**
	 * getPayment
	 *
	 * Everything should be setup, actually take the payment
	 *
	 * @param int transactions id
	 * @return mixed dependent on the gateway, value is return straight from the transaction component to user anyway
	 */
	public function getPayment($transactionId) {
		// try to create the payment, this will give us a approval url

		try {
			$this->_objects['payment']->create(
				$this->_apiContext()
			);
		}
		catch (Exception $e) {
			// fail set error message and gegt cancel url and redirect
			$this->_controller->Flash->fail(
				array(
					'title' => 'Payment Failed',
					'description' => 'There was a problem trying to setup payment, please try again or contact us to complete your order'
				)
			);

			$this->failTransaction($transactionId, 'Failed to get paypal approval link');

			return $this->_controller->redirect(
				$this->_objects['payment']->setRedirectUrls()->getCancelUrl()
			);
		}

		// set the payment id as the payment token in the transaction row
		$this->setPaymentToken($transactionId, $this->_objects['payment']->getId());

		// redirect to the approval link
		return $this->_controller->redirect(
			$this->_objects['payment']->getApprovalLink()
		);
	}

	/**
	 * return
	 *
	 * deal with a return from the gateway and check for success / fail
	 *
	 * @return array - with three elements,
	 *               - 'result' = true/false value
	 *               - 'message' = text message about transaction (i.e. reason for failing)
	 *               - 'transaction_id' = int of the transaction row
	 */
	public function processReturn() {

		debug($this->request);

	}

}
